///File : MainPanel.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
//------------------------------------------------------------------------------
#if defined USE_MULTIPLE_FILE
MainPanel::MainPanel( wxWindow* parent, MainFrame* mainFrame, DevisManager *devisManager, const int &devisId, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
#else
MainPanel::MainPanel( MainFrame* parent, DevisManager *devisManager, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
#endif
{
    // Devis Manager
    m_DevisManager = devisManager;

    #if defined USE_MULTIPLE_FILE
    m_MainFrame = mainFrame;
    m_devisId = devisId;
    #else
    m_MainFrame = parent;
    #endif

    // Controls
	wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );

	m_sizerInfo = new wxBoxSizer( wxHORIZONTAL );
	m_infoNbSection = new wxStaticText(this, wxID_ANY, _T(""));
	m_sizerInfo->Add(m_infoNbSection, 0, wxALL, 5);
	m_sizerInfo->AddSpacer(10);
	m_infoPrice = new wxStaticText(this, wxID_ANY, _T(""));
	m_sizerInfo->Add(m_infoPrice, 0, wxALL, 5);
	MainSizer->Add(m_sizerInfo);

	MainSizer->Add(new wxStaticLine(this, wxID_ANY), 0, wxEXPAND | wxALL, 5);

    wxBoxSizer* listAndBtSections = new wxBoxSizer( wxHORIZONTAL );

    m_SectionsList = new wxListBox(this, wxID_ANY);
    listAndBtSections->Add(m_SectionsList, 1, wxEXPAND | wxALL, 5);

	//wxBoxSizer* btSection = new wxBoxSizer( wxVERTICAL );
	wxGridSizer* btSection = new wxGridSizer( 2 );
	m_btAddSection = new wxButton(this, wxID_ADD);
	btSection->Add(m_btAddSection, 0, wxEXPAND | wxALL, 5);

	m_btRemoveSection = new wxButton(this, wxID_REMOVE );
    btSection->Add(m_btRemoveSection, 0, wxEXPAND | wxALL, 5);

	m_btSectionUp = new wxButton(this, wxID_UP );
    btSection->Add(m_btSectionUp, 0, wxEXPAND | wxALL, 5);

    m_btSectionDown = new wxButton(this, wxID_DOWN );
    btSection->Add(m_btSectionDown, 0, wxEXPAND | wxALL, 5);

    m_btSectionRename = new wxButton(this, wxID_EDIT );
    btSection->Add(m_btSectionRename, 0, wxEXPAND | wxALL, 5);

    listAndBtSections->Add(btSection, 0, wxEXPAND, 0);


    MainSizer->Add(listAndBtSections, 0, wxEXPAND, 0);


    MainSizer->Add(new wxStaticLine(this, wxID_ANY), 0, wxEXPAND | wxALL, 5);

    m_ScrollPanelItems = new ScrollPanelItems(this, m_DevisManager, wxID_ANY);
    MainSizer->Add(m_ScrollPanelItems, 1, wxEXPAND, 0);

	SetSizer( MainSizer );

	// Events
    //m_SectionsList->Connect( wxEVT_COMMAND_LISTBOX_SELECTED, wxCommandEventHandler(MainPanel::OnListClick), NULL, this);

    // /!\ pas de 'm_SectionsList->Connect' ==> SegmentationFault quand on quitte avec wxGTK et wx2.8.9, après avoir utilisé l'event !!
    Connect(m_SectionsList->GetId(), wxEVT_COMMAND_LISTBOX_SELECTED, wxCommandEventHandler(MainPanel::OnListClick));
    m_SectionsList->Connect( wxEVT_LEFT_DCLICK, wxEventHandler(MainPanel::OnSectionRename), NULL, this);
    m_SectionsList->Connect( wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnListKeyDown), NULL, this);

    m_btAddSection->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnAddSection), NULL, this);
    m_btRemoveSection->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnRemoveSection), NULL, this);
    m_btSectionUp->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnSectionUp), NULL, this);
    m_btSectionDown->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnSectionDown), NULL, this);
    m_btSectionRename->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(MainPanel::OnSectionRename), NULL, this);

    // Update info
    MyRefresh();
}

//------------------------------------------------------------------------------
void MainPanel::OnAddSection(wxCommandEvent& event)
{
    std::vector<std::string> stdStrSectionsList = m_DevisManager->GetAllSectionsList();

    wxArrayString wxStrSectionsList;
    wxStrSectionsList.Add(_("Other"));
    for(int i = 0; i < stdStrSectionsList.size(); i++)
    {
        wxStrSectionsList.Add(StrStdToWx(stdStrSectionsList[i]));
    }

    int choiceIndex = wxGetSingleChoiceIndex(_("Choose a section or \"Other\""), _("New section"), wxStrSectionsList, this);

    if(choiceIndex != -1)
    {
        std::string newSectionId;
        if(choiceIndex == 0) // Premier élément ==> Autre
        {
            wxString sectionIdWxStr = wxGetTextFromUser(_("Enter the section name"), _("New section"), _T(""), this);
            newSectionId = StrWxToStd(sectionIdWxStr);
        }
        else // Prédéfini
        {
            newSectionId = stdStrSectionsList[choiceIndex - 1]; // choiceIndex + 1 car on a rajouté "Autre" dans la liste
        }

        if(newSectionId != "")
        {
            m_DevisManager->NewSection(newSectionId);

            // Création de la liste pour séléction
            RefreshList();

            // Selection de la nuvelle section (derniére)
            m_SectionsList->SetSelection(m_SectionsList->GetCount() - 1);

            // Refresh general
            MyRefresh();
        }
    }
}

//------------------------------------------------------------------------------
void MainPanel::OnRemoveSection(wxCommandEvent& event)
{
    if( !( m_SectionsList->GetCount() > 0 && m_SectionsList->GetSelection() != wxNOT_FOUND))
    {
        return;
    }

    if(wxMessageBox(_("The section ans its items will be removed.\nDo you want to continue ?"), _("Remove section"), wxYES_NO | wxYES_DEFAULT | wxICON_QUESTION, this) == wxYES)
    {
        if(m_DevisManager->RemoveSection(m_SectionsList->GetSelection()))
        {
            #if defined(__WXMSW__)
            if(m_SectionsList->GetCount() != 0 && m_SectionsList->GetSelection() != wxNOT_FOUND)
            {
                m_SectionsList->SetSelection(m_SectionsList->GetCount() - 1);
            }
            #endif

            MyRefresh();
        }
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnListKeyDown(wxKeyEvent& event)
{
    if(event.GetKeyCode() == WXK_DELETE)
    {
        wxCommandEvent tmp;
        OnRemoveSection(tmp);
    }

    event.Skip();
}
//------------------------------------------------------------------------------
void MainPanel::OnSectionUp(wxCommandEvent& event)
{
    if(m_DevisManager->SwapSections(m_SectionsList->GetSelection(), m_SectionsList->GetSelection() - 1))
    {
        m_SectionsList->SetSelection(m_SectionsList->GetSelection() - 1);
        MyRefresh();
    }
}

//------------------------------------------------------------------------------
void MainPanel::OnSectionDown(wxCommandEvent& event)
{
    m_DevisManager->SwapSections(m_SectionsList->GetSelection(), m_SectionsList->GetSelection() + 1);
    {
        m_SectionsList->SetSelection(m_SectionsList->GetSelection() + 1);
        MyRefresh();
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnSectionRename(wxEvent& event)
{
    if( m_SectionsList->GetSelection() != -1)
    {
        wxString sectionIdWxStr = wxGetTextFromUser(_("Enter the section name"), _("Edit section"), StrStdToWx(m_DevisManager->GetDevisSectionName(m_SectionsList->GetSelection())), this);
        if(sectionIdWxStr != _T(""))
        {
            std::string newSectionId = StrWxToStd(sectionIdWxStr);

            if(m_DevisManager->RenameSection(m_SectionsList->GetSelection(), newSectionId))
            {
                MyRefresh();
            }

            MyRefresh();
        }
    }
}
//------------------------------------------------------------------------------

void MainPanel::RefreshList()
{
    wxWindowUpdateLocker winLock(m_SectionsList); // Bloque les redraw durant son temps de vie.

    //wxArrayString ListStrs;
    std::vector<std::string> SectionsIDs = m_DevisManager->GetSectionsList();

    //On met la liste à la bonne taille :
    while(m_SectionsList->GetCount() > SectionsIDs.size())
    {
        m_SectionsList->Delete(m_SectionsList->GetCount() - 1);
    }
    while(m_SectionsList->GetCount() < SectionsIDs.size())
    {
        m_SectionsList->Append(_T(""));
    }

    // On change les labels...
    for(int i = 0; i < SectionsIDs.size(); i++)
    {
        wxString tmp;
        tmp << StrStdToWx(SectionsIDs[i]);

        if(m_SectionsList->GetString(i) != tmp) // ...Si il le faut
        {
            m_SectionsList->SetString(i, tmp);
        }
    }
}

//------------------------------------------------------------------------------

void MainPanel::OnListClick(wxCommandEvent& event)
{
    MyRefresh();
}

//------------------------------------------------------------------------------
void MainPanel::MyRefresh()
{
    RefreshList();

    RefreshEnable();

    if(m_DevisManager->IsOpen()  && m_SectionsList->GetSelection() != wxNOT_FOUND)
    {
        m_ScrollPanelItems->SetPosSection(m_SectionsList->GetSelection());
    }
    else
    {
        m_ScrollPanelItems->ClearAll();
    }

    // Label info
    RefreshLabels();

    m_sizerInfo->Layout();
}

//------------------------------------------------------------------------------
void MainPanel::RefreshLabels()
{
    // Undo / Redo
    m_MainFrame->MyRefresh(false);

    // Label info
    if(m_DevisManager->IsOpen())
    {
        wxString label;
        label << _("Section number : ") << wxNbToStr(m_SectionsList->GetCount());
        m_infoNbSection->SetLabel(label);

        label = _T("");
        label << _("Total to pay :") << wxNbToStr(m_DevisManager->GetTotalToPay());
        m_infoPrice->SetLabel(label);
    }
    else
    {
        m_infoNbSection->SetLabel(_T(""));
        m_infoPrice->SetLabel(_T(""));
    }
}
//------------------------------------------------------------------------------
void MainPanel::RefreshEnable()
{
    // Enable si List non vide
    bool enable;
    if(m_SectionsList->GetSelection() == wxNOT_FOUND || !m_DevisManager->IsOpen())
    {
        enable = false;
    }
    else
    {
        enable = true;
    }

    m_btRemoveSection->Enable(enable);
    m_btSectionDown->Enable(enable);
    m_btSectionUp->Enable(enable);
    m_btSectionRename->Enable(enable);
    m_ScrollPanelItems->EnableBt(enable);

    // Enable si open
    m_SectionsList->Enable(m_DevisManager->IsOpen());
    m_btAddSection->Enable(m_DevisManager->IsOpen());
}
//------------------------------------------------------------------------------
#if defined USE_MULTIPLE_FILE
int MainPanel::GetDevisId()
{
    return m_devisId;
}
#endif
